/** Source file of framegrabber class concerning the ITI MVC 150/40 framegrabber	*/
/** Author: Peter Hauge																*/
/** Version 0.1																		*/

// If compiling with Visual C++, we remove some warnings
#ifdef _WIN32
#pragma warning(disable: 4786) // symbol greater than 255 character
#endif /* _WIN32 */

#include "ITIMVC150_40.h"
#include <itxcore.h>
#include <icp.h>

namespace ipl{ // use namespace in C++

using std::cerr;
using std::endl;

CITIMVC150_40::CITIMVC150_40()
{
	// Initialization of member variables
	MODCNF* m_PcVisionModule = NULL;
	m_AcquisitionModuleFrame = ICP_BAD_ARG; // To make sure that the return value is assigned to this varible in Initialize()
	// call base class constructor
	CFrameGrabberGeneric::CFrameGrabberGeneric();
}


CITIMVC150_40::~CITIMVC150_40()
{
	Close();
	// call base class destructor
	CFrameGrabberGeneric::~CFrameGrabberGeneric();
}


bool CITIMVC150_40::Initialize(string cfgFileName)
{	
	unsigned short width, height;
	char* fileName = new char[cfgFileName.length()+1];
	strcpy(fileName,cfgFileName.c_str());
	if(stricmp(fileName,cfgFileName.c_str()))
	{
		cerr << "Error: strcpy failed to copy in CITIMVC150_40::Initialize()" << endl;
		return false;
	}
	// Load configuration file for camera and grabber
    if(itx_load_cnf(fileName) != ITX_NO_ERROR) 
	{
		cerr << "Error: Unable to load configuration file " << fileName << endl;
		return false;
    }
	delete[] fileName;

    // initialize system 0 boards and their daughter boards 
    if(itx_init_sys(0) != ITX_NO_ERROR)
	{
		cerr << "Error: Unable to initialize system" << endl;
		itx_remove_sys(0);
		return false;
    }

    // Get a MODCNF pointer to the PCVision board 
	if((m_PcVisionModule = itx_get_modcnf(0,"ICP",(short)(0))) == NULL)
	{
		cerr << "Error: No PCVision Module installed" << endl;
		itx_remove_sys(0);
		return false;
    }

	// Get dimensions of acquisition module
	short t1 = icp_get_acq_dim(m_PcVisionModule,&width,&height);
	if(t1 == ICP_BAD_ARG)
	{
		cerr << "Error: PCVision Module not defined" << endl;
		itx_remove_sys(0);
		return false;
    }
	else if(t1 == ICP_BAD_DMODCNF)
	{
		cerr << "Error: Acquisition Module not defined" << endl;
		itx_remove_sys(0);
		return false;
    }
	// set the attributes and given parameter dimensions
	m_Width = width;
	m_Height = height;
	m_DimensionsAvailable=true;

    // Get a MODCNF pointer to Acquisition Module
	if((m_AcquisitionModuleFrame = icp_create_frame(m_PcVisionModule,width,height,ICP_PIX8,ICP_MONO)) == ICP_BAD_ARG)
	{
		cerr << "Error: Bad arguments in call of icp_create_frame()" << endl;
		return false;
	}
	return Open();
}


bool CITIMVC150_40::Close()
{
	if(itx_remove_sys(0) != ITX_NO_ERROR)
	{
		return false;
	}
	else
	{
		return CFrameGrabberGeneric::Close();
	}
}

bool CITIMVC150_40::Acquire(CStdImage& Image, CCameraGeneric& Camera)
{
	// Acquire image into am_frame
	short t2 = icp_snap(m_PcVisionModule,m_AcquisitionModuleFrame);
	if(t2 == ICP_BAD_ARG)
	{
		cerr << "Error: Invalid PCVision Module or Acquisition Module Frame" << endl;
		return false;
	}
	else if(t2 == ICP_OUT_FB_MEM)
	{
		cerr << "Error: Addressing outside frame" << endl;
		return false;
	}
	else if(t2 == ITX_TIMEOUT)
	{
		cerr << "Error: Timeout (snap)" << endl;
		return false;
	}

	unsigned long* data = (unsigned long*)malloc(m_Width*m_Height*sizeof(char));
	// Allocate memory if necessary
	if ((Image.GetWidth()!=m_Width) || (Image.GetHeight()!=m_Height) || (Image.GetBits()!=8) ||
		(Image.GetBorderSize()!=0))
	{
		Image.AllocFast(m_Width,m_Height,ICP_PIX8);
	}

	// Read area from am_frame into data
	short t3 = icp_read_area(m_PcVisionModule,m_AcquisitionModuleFrame,0,0,m_Width,m_Height,data);
	if(t3 != ICP_NO_ERROR)
	{
		cerr << "Error: Failed to read from PCVsion Module memory to host memory" << endl;
		return false;
	}

	char* data2=(char*)Image.GetTImagePtr()->pPixelStream;
	for(int vOffset = 0; vOffset < m_Height; vOffset++)
		memcpy(&(data2)[vOffset*m_Width],&((char*)data)[(m_Height-vOffset-1)*m_Width],m_Width);

	free(data);
	return true;
}

/*bool CITIMVC150_40::Acquire(CStdImage& Image, CCameraGeneric& Camera)
{
	// Acquire image into am_frame
	short t2 = icp_snap(m_PcVisionModule,m_AcquisitionModuleFrame);
	if(t2 == ICP_BAD_ARG)
	{
		cerr << "Error: Invalid PCVision Module or Acquisition Module Frame" << endl;
		return false;
	}
	else if(t2 == ICP_OUT_FB_MEM)
	{
		cerr << "Error: Addressing outside frame" << endl;
		return false;
	}
	else if(t2 == ITX_TIMEOUT)
	{
		cerr << "Error: Timeout (snap)" << endl;
		return false;
	}

	unsigned long* data = (unsigned long*)malloc(m_Width*m_Height*sizeof(char));
	unsigned long* data2 = (unsigned long*)malloc(m_Width*m_Height*sizeof(char));

	// Read area from am_frame into data
	short t3 = icp_read_area(m_PcVisionModule,m_AcquisitionModuleFrame,0,0,m_Width,m_Height,data);
	if(t3 != ICP_NO_ERROR)
	{
		cerr << "Error: Failed to read from PCVsion Module memory to host memory" << endl;
		return false;
	}

	for(int vOffset = 0; vOffset < m_Height; vOffset++)
		memcpy(&((char*)data2)[vOffset*m_Width],&((char*)data)[(m_Height-vOffset-1)*m_Width],m_Width);

	//img = COSSImage(m_Width,m_Height,ICP_PIX8);
	Image.Alloc(m_Width,m_Height,ICP_PIX8);
	memcpy(Image.GetImageDataPtr(),(char*)data2,m_Width*m_Height);
	free(data);
	free(data2);
	return true;
}*/

/*bool CITIMVC150_40::Acquire(COSSImage& img)
{
	// Acquire image into am_frame
	short t2 = icp_snap(m_PcVisionModule,m_AcquisitionModuleFrame);
	if(t2 == ICP_BAD_ARG)
	{
		cerr << "Error: Invalid PCVision Module or Acquisition Module Frame" << endl;
		return false;
	}
	else if(t2 == ICP_OUT_FB_MEM)
	{
		cerr << "Error: Addressing outside frame" << endl;
		return false;
	}
	else if(t2 == ITX_TIMEOUT)
	{
		cerr << "Error: Timeout (snap)" << endl;
		return false;
	}

	unsigned long* data = (unsigned long*)malloc(m_Width*m_Height*sizeof(char));
	unsigned long* data2 = (unsigned long*)malloc(m_Width*m_Height*sizeof(char));

	// Read area from am_frame into data
	short t3 = icp_read_area(m_PcVisionModule,m_AcquisitionModuleFrame,0,0,m_Width,m_Height,data);
	if(t3 != ICP_NO_ERROR)
	{
		cerr << "Error: Failed to read from PCVsion Module memory to host memory" << endl;
		return false;
	}

	for(int vOffset = 0; vOffset < m_Height; vOffset++)
		memcpy(&((char*)data2)[vOffset*m_Width],&((char*)data)[(m_Height-vOffset-1)*m_Width],m_Width);

	img = COSSImage(m_Width,m_Height,ICP_PIX8);
	memcpy(img.GetImageDataPtr(),(char*)data2,m_Width*m_Height);
	free(data);
	free(data2);
	return true;
}*/

} // end namespace ipl
