#ifndef _IPL98_CAMERAGENERIC_H
#define _IPL98_CAMERAGENERIC_H

//#include <ipl98_types.h>
//#include <image.h>
#include <string>

namespace ipl{ // use namespace if C++

using std::string;

/** CCameraGeneric is an abstract base class from which all
	camera classes are derived, last updated 9/25/2002. A Camera class
	is the only class needed to control shutter, gain, zoom and partial scan
	for the camera and grabbing an image. More advanced features should
	be found in the inherited classes. A frame grabber class
	attribute may be available in inherited classes to control
	specific frame grabber functions. This class is not complete
	and a lot of changes in near future must be expected.
	At the moment no derived classes exists.

	@version 0.73,  by edr@mip.sdu.dk
	@author Ren Dencker Eriksen */

class CCameraGeneric{
	protected: // attributes
		/// Gain factor
		double m_GainValue;
		/// Gain min and max values
		double m_GainMin, m_GainMax;
		/// Shutter value in micro-seconds
		double m_ShutterValue;
		/// Shutter min and max values in micro seconds
		double m_ShutterMin, m_ShutterMax;
		/// Image width and height
		unsigned int m_Width, m_Height;
		/** True when Initialize() or another derived class implementation of  a Initialize(..)
			method succeeds. IsInitialized() can be used to check this value. */
		
	public: // methods
	
		bool m_Initialized;
		
		CCameraGeneric(){}; // default constructor
		/** Initialize the camera. Derived classes may have other initializing methods!
			@return False, if initialization failed. */
		virtual bool Initialize()=0;
		/** Returns true if this class has been succesfully intialized by a previous call to Initialize().
			@return True, if this class is initialized. */
		inline bool IsInitialized() const;
		/** Closes devices etc. if necessary, no need to overwrite if your derived class
			need not to close any devices. Sets m_Initialized to false.
			@return The dummy implementation always returns true, see derived class for
				specific return information. */
		virtual bool Close();
		/** Return the name and model of the camera.
			@return Name and model. */
		virtual string GetCameraName() const=0;
		/** Check if shutter is available.
			@return True if shutter is available */
		virtual bool ShutterAvailable() const=0;
		/** Get actual shutter value.
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return shutter value in micro-seconds. */
		virtual double GetShutter() const;
		/** Set shutter value. If the given value is not possible the nearest
			value are choosen.
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@param Value New shutter value.
			@return New nearest shutter value. */
		virtual double SetShutter(double Value);
		/** Increment shutter value to next possible value.
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return New shutter value, same as previous if max. value reached */
		virtual double IncShutter();
		/** Decrement shutter value to next possible value.
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return New shutter value, same as previous if min. value reached */
		virtual double DecShutter();
		/** Set the shutter time to the previous value times two
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return New Shutter value, same as previous if max. value reached */
		virtual double ShutterDouble();
		/** Set the shutter time to the previous value divided by two
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return New Shutter value, same as previous if min. value reached */
		virtual double ShutterHalf();
		/** Get min. shutter value in micro-seconds.
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Min. shutter value. */
		virtual double GetShutterMin() const;
		/** Get max. shutter value in micro-seconds.
			Note: If shutter is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Max. shutter value. */
		virtual double GetShutterMax() const;
		/** Check if gain is available.
			@return True if zoom is available */
		virtual bool GainAvailable() const=0;
		/** Get actual gain value.
			Note: If gain is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Gain value. */
		virtual double GetGain() const;
		/** Set gain value. If the given value is not possible the nearest
			value are choosen.
			Note: If gain is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@param Value New gain value.
			@return New nearest gain value. */
		virtual double SetGain(double Value);
		/** Increment gain value to next possible value.
			Note: If gain is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return New gain value, same as previous if max. value reached */
		virtual double IncGain();
		/** Decrement gain value to next possible value.
			Note: If gain is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return New gain value, same as previous if min. value reached */
		virtual double DecGain();
		/** Get min. gain value.
			Note: If gain is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Min. gain value. */
		virtual double GetGainMin() const;
		/** Get max. gain value.
			Note: If gain is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Max. gain value. */
		virtual double GetGainMax() const;
		/** Check if zoom is available.
			@return True if zoom is available */
		virtual bool ZoomAvailable() const=0;
		/** Get actual zoom value.
			Note: If zoom is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Zoom value. */
		virtual double GetZoom() const;
		/** Set zoom value. If the given value is not possible the nearest
			value are choosen.
			Note: If zoom is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@param Value New zoom value.
			@return New nearest zoom value. */
		virtual double SetZoom(double Value);
		/** Get min. zoom value.
			Note: If zoom is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Min. zoom value. */
		virtual double GetZoomMin() const;
		/** Get max. zoom value.
			Note: If zoom is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return Max. zoom value. */
		virtual double GetZoomMax() const;
		/** Check if partial scan is available.
			Note: If partial scan is not available then a dummy implementation
			returning -1 is used and an error message is produced.
			@return True if partial scan is available */
		virtual bool PartialScanAvailable() const=0;
		/** Get partial scan positions.
			Note: If partial scan is not available then a dummy implementation
			returning false is used and an error message is produced.
			@param UpperLeft Position in upper left corner of image.
			@param LowerRight Position in lower left corner of image.
			@return False if reading the positions failed. */
		
		/*
		virtual bool GetPartialScan(CPoint2D<UINT16>& UpperLeft, CPoint2D<UINT16>& LowerRight) const;
		virtual bool SetPartialScan(const CPoint2D<UINT16>& UpperLeft,const CPoint2D<UINT16>& LowerRight);
		
	   */
	 
		virtual unsigned int GetWidth() const{ return m_Width;}
		/** Get the maximum height of image.
			@return Height in pixels. */
		virtual unsigned int GetHeight() const{ return m_Height;}
		/** Acquire image from camera. May need a void* to image data as parameter??
			@return false if grabbing failed */
		
		//virtual bool Acquire(CStdImage& Image)=0;
		/** Acquire image from camera. This implementation simply calls Acquire(CStdImage)
			if the derived class has not overwritten this method.
			@return false if grabbing failed */
		//virtual bool Acquire(CImage& Image){return Acquire((CStdImage&)Image);}
		/** Acquire image from camera. This implementation simply calls Acquire(CStdImage)
			if the derived class has not overwritten this method.
			@return false if grabbing failed */
		//virtual bool Acquire(CByteImage& Image){return Acquire((CByteImage&)Image);}
		/** Loads data from given path and filename into this class. Derived classes
			can overload this method, if camera supports setting parameters from
			computer. This dummy implementation simply returns true.
			@param Start If true (default), the class will initialize the camera and set parameters
				for camera. See SetWithCurrentValue() for more info.
			@return True, always. This is a dummy implementation, see derived classes for more info. 
			@version 0.5 */
		virtual bool Load(const char* pPathAndFileName, bool Start=true) throw(string);
};

/////////////////////////////////////////////////
//// Inline methods
/////////////////////////////////////////////////

inline bool CCameraGeneric::IsInitialized() const
{
	return m_Initialized;
}

} // end namespace ipl

#endif //_IPL98_CAMERAGENERIC_H